//      t: Time, passed from PSIM by value
//   delt: Time step, passed from PSIM by value
//     in: input array, passed from PSIM by reference
//    out: output array, sent back to PSIM (Note: the values of out[*] can
//         be modified in PSIM)

// The maximum length of the input and output array "in" and "out" is 30.

// Las entradas del sistema son
// in[0]= fr;       frecuencia de la red
// in[1]= Vref;     tension de referencia sin normalizar
// in[2]= fs;       frecuencia de conmutacin: La frecuencia de muestreo es fm=fs
// in[3]= Vcc;      tension de la barra CC
// in[4]= vo;       tension de salida en bornes del capacitor del filtro LC
// in[5]= iL;       corriente del inductor del filtro LC
// in[6]= t_rampa;  tiempo final de la rampa de variacin de la referencia

#include "ganancias_inversor.h"

// Aqui se implementa una estrategia de control diseada en matlab, que consta de 
// un compensador proporcional para el lazo interno de control de corriente y de
// un compensador PI para el lazo externo de control de tensin.
// La frecuencia de muestreo de ambos lazos es de 10 kHz.

void __declspec(dllexport) simuser (double t, double delt, double *in, double *out)

{
   	static double vo=0;   // tension de fase en bornes del capacitor de filtro
   	static double iL=0;   // Corriente en el inductor.
   	static double uk=0,  upk=0, updk=0, uc=0;
   	
   	//!!!!!!!!!!!DEFINIR LAS VARIABLES QUE FALTAN
   	
   	static double ek=0, ekm1=0, ekm2=0;
   	static double ref, rd=0, ri=0;
	static double rampa=0;
	static double nc=5;	  // N de periodos de la rampa
	static double muestra;
			
	// Variables enteras
	static int contador = 0, Ncont, calculo_u, muestreo, direccion = 1;
	static int PWM1, PWM2;
	static int ComparReg1, ComparReg2;
	static int i=0, j=0;
	static int samp=0;
	
	static double Vcc;
	Vcc=in[3];
	static double fn;
	fn=Vbase/Vcc;            /* Factor de normalizacin de la accin de control
							 /* aplicada a la planta */
	
	static double fr;        /* Frecuencia fundamental de la tensin de salida */
	fr=in[0];
	static double Ts;        /* Periodo de muestreo */
	Ts = 0.0001;
	static double Tr;        /* Periodo fundamental de la tensin de salida */
	Tr=1/fr;
	
// Se calcula el numero de conteos en un periodo: Valor mximo del contador
   Ncont=Ts/delt;
   muestra=0;

   if (calculo_u==1)
      {
      calculo_u=0;
	  
	  // Poner una senoidal con la amplitud deseada para la tension de referencia
      ref = in[1]/Vbase;  // in[1]/Vbase Agregar este trmino para implementacin en p.u.
      
      /* Variacin inicial de Vref (generacin de rampa):
	  rampa = t;
      if (t < nc*Tr)
           {rd = (1/(nc*Tr))*rampa*ref;} // Calcula incremento de referencia.
      else
      	  {rd = ref;}
      */
	  	  
      rd = ref;  // Se aplica la referencia sin variacin de amplitud en rampa
 	  /*********************************************************************************/
      /*                         Se calcula el error de tension                        */
      /*********************************************************************************/
      ek = rd - vo;
      /*********************************************************************************/
      /*                   Calculo de la estrategia de control                         */
      /*********************************************************************************/
      // Lazo de control de tensin con controladores resonantes + compensador PD Pred
      // upd(km) = K1*e(km-1) + K2*e(km-2);
    
      updk = K1*ekm1 + K2*ekm2;
      
      //!!!!!!!ECUACIONES RECURSIVAS FALTANTES
      
      uk = updk + umi_1k + umi_3k; //+ umi_5k + umi_7k + umi_9k + umi_11k; 
	  
	  // Actualizacin de las variables anteriores
	  ekm2 = ekm1;
	  ekm1 = ek;  
	  
	  umi_1km2=umi_1km1;
	  umi_1km1=umi_1k;
	  umi_3km2=umi_3km1;
	  umi_3km1=umi_3k;
	  	  
	  //Para lazo abierto
      //uk = rd; 

      uc=uk*(Ncont/4)*(Vbase/Vcc);  //(Vbase/Vcc) Agregar este trmino para implementacin en p.u.
      } //termina el calculo de la estrategia de control


     //**********************************************************//
	 // Incrementamos o decrementamos el contador segun corresponda
     contador = contador + direccion;
   if (contador == 0)
      {
	  direccion=1;
	  }
   if (contador == Ncont/2)
	  {
	  direccion=-1;
	  muestreo=1;
      muestra=1;
	  }

   if (contador == Ncont/2)
      {
      ComparReg1 =  uc + Ncont/4;
      ComparReg2 = -uc + Ncont/4;
      }

   // Maquina de estado que define el patron PWM
    if (ComparReg1 > contador)
        PWM1=1;
    else
        PWM1=0;

    if (ComparReg2 > contador)
        PWM2=1;
    else
        PWM2=0;

    if(muestreo == 1)
    	{
       // Se realiza el muestreo de las variables controladas
        vo = in[4]/Vbase;   // in[4]/Vbase Agregar este trmino para implementacin en p.u.
        //iL = in[5]/Ibase;   // in[5]/Ibase Agregar este trmino para implementacin en p.u.
        //vo = -vo;	        // Comentar estas dos lneas siguientes, si se invierten las seales
        //iL = -iL;	        // de disparo que van a las llaves superior e inferior de cada brazo, o quitar el signo menos.
        calculo_u = 1;
      }

// Se actualizan los valores de las salidas
	out[0] = PWM1; //brazo 1
	out[1] = PWM2; //brazo 2
	out[2] = uk;
	out[3] = rd;
	out[4] = umi_1k;
	out[5] = ek;
	out[6] = muestra*500;
	out[7] = contador;
	out[8] = calculo_u;
	
	muestreo=0;
}
