% DEFINICION DE PARAMETROS DE SIMULACION
% clc;
clear; 
close all
% PARAMETROS PARA GRAFICAR EL BODE EN Hz
P = bodeoptions; 
P.FreqUnits = 'Hz'; 
P.grid='on';
format long

fr=50;					% Frecuencia deseada de la tensin de salida en Hz
wo=2*pi*fr;             % Frecuencia angular en rad/seg
fm=10000;               % Frecuencia de muestreo
Tm=1/fm;                % Periodo de muestreo

% Parametros del convertidor CC-CA
s=tf('s');
z=tf('z',Tm);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARAMETROS DE LA PLANTA.
L=150e-6;              % Inductancia en uH
C=240e-6;          	% Capacitancia en uF
rL=0.25;                % Resistencia serie del inductor en Ohm
Vbase=311;
Gp=1/(L*C*s*s+rL*C*s+1);            % DEFINIR LA PLANTA
step(Gp)
% Funcion de transferencia de la planta en tiempo discreto
Gpd= c2d(Gp,Tm);

% Diseo de los controladores resonantes
% Resonante en la frecuencia de la fundamental
Gr1 = s/(s^2 + wo^2);

% Resonantes en las armonicas de la fundamental
Gr3 = s/(s^2 + (3*wo)^2);

% Controladores resonantes en tiempo discreto
Grd1=c2d(Gr1,Tm,'zoh');      %50 Hz        
Grd3=c2d(Gr3,Tm,'zoh');      %150 Hz 

% Ganancias controladores resonantes
kmi_arm = 10;
kmi_1 = 200;          %50 Hz
kmi_3 = kmi_arm;      %150 Hz

Gmi1 = kmi_1*Grd1;
Gmi3 = kmi_3*Grd3;

% Coeficientes de los controladores resonantes
[num,den]=tfdata(Gmi1,'v');
a2_1 = num(1);
a1_1 = num(2);
a0_1 = num(3);
b1_1 = den(2);
b0_1 = den(3);

[num,den]=tfdata(Gmi3,'v');
a2_3 = num(1);
a1_3 = num(2);
a0_3 = num(3);
b1_3 = den(2);
b0_3 = den(3);
   

% Analisis de desempeo y estabilidad
% Funcion de transferencia de lazo abierto: Gmi1 * Gpd
Gla = minreal(series(Gmi1,Gpd));
figure; margin(Gla)
legend('Estabilidad Gpd * Gmi1')
figure; pzmap(Gla);

% Funcion de trasnferencia de LC con resonante fundamental
Glc = minreal(feedback(Gla,1));
figure; bode(Glc,P)
legend('Bode LC Gpd * Gmi1')
figure; pzmap(Glc);

t=0:Tm:0.04*3;
r=sin(2*pi*fr*t);
figure; lsim(Glc,r,t);
legend('Rta senoidal a LC Gpd * Gmi1')
vo=lsim(Glc,r,t);
ev=r' - vo;
figure; plot(t,ev)
legend('Error de Tensin')
% close all

% Ganancias del PD predictivo
K1=-0.025;
K2=0.011;

zc=K2/K1;
GcPD=K1*(z - zc)/z^2;
Gla_PD = series(GcPD,Gpd);
figure; margin(Gla_PD)
Glc_PD = feedback(Gla_PD,1);
figure; bode(Glc_PD,P);
figure; step(Glc_PD)

Gres=Gmi1 + Gmi3; %+ Gmi5 + Gmi7 + Gmi9 + Gmi11;
Gcc=1*GcPD + Gres; 
Gla_PMI = minreal(series(Gcc,Gpd));
figure; pzmap(Gla_PMI)
figure; margin(Gla_PMI)
legend('Estabilidad Gpd + Resonantes')

Glc_PMI = minreal(feedback(Gla_PMI,1));
P.PhaseWrapping = 'on';
figure; bode(Glc_PMI,P)
legend('Bode LC Gpd + Resonantes')
figure; step(Glc_PMI)
legend('Rta escaln a LC Gpd + Resonantes')
figure; lsim(Glc_PMI,r,t);
legend('Rta senoidal a LC Gpd + Resonantes')
vo=lsim(Glc_PMI,r,t);
ev=r' - vo;
figure; plot(t,ev)
% axis([0 max(t) -1 1])
legend('Error de Tensin')
% close all


% Guarda los datos en un archivo .TXT para usar en el DevC++
fid = fopen('ganancias_inversor.h','w');

fprintf(fid, '#define a2_1 %1.12f \r\n',a2_1);
fprintf(fid, '#define a1_1 %1.12f \r\n',a1_1);
fprintf(fid, '#define a0_1 %1.12f \r\n',a0_1);
fprintf(fid, '#define b1_1 %1.12f \r\n',b1_1);
fprintf(fid, '#define b0_1 %1.12f \r\n',b0_1);

fprintf(fid, '#define a2_3 %1.12f \r\n',a2_3);
fprintf(fid, '#define a1_3 %1.12f \r\n',a1_3);
fprintf(fid, '#define a0_3 %1.12f \r\n',a0_3);
fprintf(fid, '#define b1_3 %1.12f \r\n',b1_3);
fprintf(fid, '#define b0_3 %1.12f \r\n',b0_3);

% fprintf(fid, '#define a2_5 %1.12f \r\n',a2_5);
% fprintf(fid, '#define a1_5 %1.12f \r\n',a1_5);
% fprintf(fid, '#define a0_5 %1.12f \r\n',a0_5);
% fprintf(fid, '#define b1_5 %1.12f \r\n',b1_5);
% fprintf(fid, '#define b0_5 %1.12f \r\n',b0_5);
% 
% fprintf(fid, '#define a2_7 %1.12f \r\n',a2_7);
% fprintf(fid, '#define a1_7 %1.12f \r\n',a1_7);
% fprintf(fid, '#define a0_7 %1.12f \r\n',a0_7);
% fprintf(fid, '#define b1_7 %1.12f \r\n',b1_7);
% fprintf(fid, '#define b0_7 %1.12f \r\n',b0_7);
% 
% fprintf(fid, '#define a2_9 %1.12f \r\n',a2_9);
% fprintf(fid, '#define a1_9 %1.12f \r\n',a1_9);
% fprintf(fid, '#define a0_9 %1.12f \r\n',a0_9);
% fprintf(fid, '#define b1_9 %1.12f \r\n',b1_9);
% fprintf(fid, '#define b0_9 %1.12f \r\n',b0_9);
% 
% fprintf(fid, '#define a2_11 %1.12f \r\n',a2_11);
% fprintf(fid, '#define a1_11 %1.12f \r\n',a1_11);
% fprintf(fid, '#define a0_11 %1.12f \r\n',a0_11);
% fprintf(fid, '#define b1_11 %1.12f \r\n',b1_11);
% fprintf(fid, '#define b0_11 %1.12f \r\n',b0_11);
% 
% fprintf(fid, '#define a2_13 %1.12f \r\n',a2_13);
% fprintf(fid, '#define a1_13 %1.12f \r\n',a1_13);
% fprintf(fid, '#define a0_13 %1.12f \r\n',a0_13);
% fprintf(fid, '#define b1_13 %1.12f \r\n',b1_13);
% fprintf(fid, '#define b0_13 %1.12f \r\n',b0_13);
% 
% fprintf(fid, '#define a2_15 %1.12f \r\n',a2_15);
% fprintf(fid, '#define a1_15 %1.12f \r\n',a1_15);
% fprintf(fid, '#define a0_15 %1.12f \r\n',a0_15);
% fprintf(fid, '#define b1_15 %1.12f \r\n',b1_15);
% fprintf(fid, '#define b0_15 %1.12f \r\n',b0_15);
% 
% fprintf(fid, '#define a2_17 %1.12f \r\n',a2_17);
% fprintf(fid, '#define a1_17 %1.12f \r\n',a1_17);
% fprintf(fid, '#define a0_17 %1.12f \r\n',a0_17);
% fprintf(fid, '#define b1_17 %1.12f \r\n',b1_17);
% fprintf(fid, '#define b0_17 %1.12f \r\n',b0_17);
% 
% fprintf(fid, '#define a2_19 %1.12f \r\n',a2_19);
% fprintf(fid, '#define a1_19 %1.12f \r\n',a1_19);
% fprintf(fid, '#define a0_19 %1.12f \r\n',a0_19);
% fprintf(fid, '#define b1_19 %1.12f \r\n',b1_19);
% fprintf(fid, '#define b0_19 %1.12f \r\n',b0_19);

fprintf(fid, '#define K1 %1.12f \r\n',K1);
fprintf(fid, '#define K2 %1.12f \r\n',K2);

fprintf(fid, '#define Vbase %1.12f \r\n', Vbase);

fclose(fid);